#!/usr/bin/env node
const chalk = require("chalk");
const packageInfo = require('../package.json');
const commander = require('commander');
const collection = require('../lib/collection');
const translate = require('../lib/translate');
const createLang = require('../lib/createLang');
const initTranslateConf = require('../lib/init');
const clear = require('../lib/clear');

commander.version(packageInfo.version, '-v, --version');

commander.command('collect [src]')
  .description('对src目录下的vue/js文件进行中文提取并生成国际化资源文件，默认src为执行目录下的src目录')
  .option('-p, --filepath <filepath>', '设置国际化文件的路径，默认为执行目录下的src/lang/locale目录，请务必设置一个单独的目录来放置国际化资源文件')
  .option('-f, --filename <filename>', '设置生成文件的文件名，默认为 zh，会自动添加.js 后缀')
  .action((src = 'src', { filepath = 'src/lang/locale', filename = 'zh' }) => {
    collection(src, { filepath, filename });
  });

commander.on('command:*', function () {
  console.error('无效命令: %s\n输入 --help 查看命令列表.', commander.args.join(' '));
  process.exit(1);
});


commander.command('init')
  .description('初始化翻译配置项')
  .action(() => {
    initTranslateConf();
  })

commander.command('translate [code]')
  .description('开始启用自动翻译')
  .option('-f, --file <filename>', '选择翻译的文件')
  .action((code = 'en', { file }) => {
    translate(code, file);
  })

commander.command('clear [src]')
  .description('清除未再使用的语言包')
  .action((src = 'src') => {
    clear(src);
  })
commander.command('create')
  .description('创建新语言包')
  .action((target) => {
    if (target) {
      createLang(target);
    } else {
      console.log(chalk.red('请填写需要创建的语言包名称'))
    }
  })

if (process.argv.length === 2) {
  commander.help();
}

commander.parse(process.argv);
