const loaderUtils = require("loader-utils");
const path = require("path");
const fs = require("fs");
const FileProcess = require("./lib/fileProcess");
// 默认配置
const defaultLoaderConfig = {
	options: {
		localeFile: path.join(process.cwd(), "src/locale/zh_cn.js"),
		target: "@/lang", // 定义 i18n 的文件路径
		localKey: "",
		whitePathList: [],
	},
	local: null, // 语言包对象
};

/**
 * @description: template 内容替换
 * @author: - wsz -
 * @param {*} content 文件的内容
 * @param {*} options 参数
 */
const replaceTemplateContent = (content) => {
	return FileProcess.generateTemplate(defaultLoaderConfig.local, content, true);
};

/**
 * @description: script 内容替换
 * @author: - wsz -
 * @param {*} content 文件的内容
 * @param {*} options 参数
 */
const replaceScriptContent = (content) => {
	return FileProcess.generateScript(defaultLoaderConfig.local, content, true, defaultLoaderConfig.options);
};

// 读取local 语言数据
const setupLocalMessage = ({ localeFile, localKey }) => {
	// 不存在语言包数据，但是传入了文件地址，进行读取
	if (!defaultLoaderConfig.local && fs.existsSync(localeFile)) {
		const localFileData = require(localeFile);
		// 定义了 数据文件的key 直接读取key对应的数据
		if (localKey && localFileData.hasOwnProperty(localKey)) {
			Object.assign(defaultLoaderConfig, {
				local: localFileData[localKey],
			});
		} else {
			Object.assign(defaultLoaderConfig, {
				local: localFileData,
			});
		}
		// 文件变化监听
		const isDev = process.env.NODE_ENV !== "production";
		if (isDev) {
			fs.watchFile(
				localeFile,
				{
					interval: 1000,
				},
				() => {
					// 清理缓存
					delete require.cache[require.resolve(localeFile)];
					defaultLoaderConfig.local = null;
				}
			);
		}
	}
};

// 导出一个函数供webpack使用
module.exports = function (source) {
	const options = loaderUtils.getOptions(this);

	Object.assign(defaultLoaderConfig.options, options);
	const localPathDir = path.parse(defaultLoaderConfig.options.localeFile).dir;
	if (!defaultLoaderConfig.options.whitePathList.includes(localPathDir)) {
		defaultLoaderConfig.options.whitePathList.push(localPathDir);
	}

	setupLocalMessage(defaultLoaderConfig.options);

	// 没有定义语言包数据。返回
	if (!defaultLoaderConfig.local) return source;

	if (this.resourcePath.indexOf("node_modules") > -1) return source;

	let result = "";
	const fileSuffix = path.extname(this.resourcePath);

	const isLocalPath = defaultLoaderConfig.options.whitePathList.includes(path.parse(this.resourcePath).dir);
	if ([".js", ".jsx", ".ts", ".tsx"].includes(fileSuffix) && !isLocalPath) {
		// 处理js文件
		result = replaceScriptContent(source);
	} else if (fileSuffix === ".vue") {
		// 处理vue文件 的 template 部分
		result = source.replace(/(<template[^>]*>)((.|\n)*)(<\/template>)/gim, (_, preTag, content, $3, afterTag) => {
			return `${preTag}${replaceTemplateContent(content)}${afterTag}`;
		});
		// 处理vue文件的 script 部分
		result = result.replace(/(<script[^>]*>)((.|\n)*)(<\/script>)/gim, (_, preTag, content, $3, afterTag) => {
			const replaceContent = replaceScriptContent(content);
			return `${preTag}${replaceContent}${afterTag}`;
		});
	} else {
		// local 资源目录直接返回
		result = source;
	}
	return result;
};
