
const path = require("path");
const fs = require("fs");
const ora = require("ora");
const { configFileName } = require("./const");

const Utils = require("./utils");
const FileProcess = require("./fileProcess");
const chalk = require("chalk");
const locals = {};
let sourceLocal = {}
let rootPath = null;
const isObject = (obj) => Object.prototype.toString.call(obj) === '[object Object]'

const isCN = (str) => /([\u4e00-\u9fa5])+/.test(str)
const collectVueLocal = (file) => {
	const content = fs.readFileSync(file, "utf8");
	// // 获取模板部分
	const [, templateContent = ""] = content.match(/<template[^>]*>((.|\n)*)<\/template>/im) || [];
	FileProcess.generateTemplate(locals, templateContent);
	// 获取script部分
	const [, scriptContent = ""] = content.match(/<script[^>]*>((.|\n)*)<\/script>/im) || [];
	FileProcess.generateScript(locals, scriptContent);
};

const collectJsLocal = (file) => {
	const content = fs.readFileSync(file, "utf8");
	FileProcess.generateScript(locals, content);
};

module.exports = async function (src) {
	const spinner = ora(chalk.blue("➤ 开始清理语言包，请稍后......\n"));
	spinner.start();
	let processFile = "";
	let oldLocals = {}; // 中文数据
	const delKey = {}; // 要删除的key
	const delKeyCache = {}; // 要删除的key
	let translateConfig = {};
	// 检测有无初始化配置文件
	if (fs.existsSync(configFileName)) {
		processFile = path.join(process.cwd(), configFileName);
		translateConfig = require(processFile);
	} else {
		spinner.fail(chalk.red(` 💔 ${configFileName}文件不存在，请先初始化项目`));
		return;
	}
	// 读取中文文件
	if (fs.existsSync(translateConfig.dir + translateConfig.file)) {
		processFile = path.join(process.cwd(), translateConfig.dir + translateConfig.file);
		sourceLocal = require(processFile);
		if (!sourceLocal.hasOwnProperty('local')) {
			//判断是否存在key
			spinner.fail(chalk.red(` 💔 语言包中不存在 local字段`));
			return
		}
		oldLocals = sourceLocal.local;
	} else {
		spinner.fail(chalk.red(` 💔 中文语言包不存在,${translateConfig.dir + translateConfig.file}\n`));
		return;
	}

	const langList = []; // 已有的语言列表  如 ['en']
	try {
		//   获取所有的需要导入的语言
		fs.readdirSync(translateConfig.dir).forEach((file) => {
			const lang = file.split(".")[0];
			if (file.indexOf("js") !== -1) {
				langList.push(lang);
			}
		});
	} catch (e) {
		console.log(e);
	}
	// ----- 重新抓取中文，得到在使用得语言包 locals -----
	rootPath = path.join(process.cwd(), src);
	Utils.getAllFiles(rootPath).forEach((file) => {
		if (file.indexOf(translateConfig.dir) < 0) {
			path.extname(file) === ".vue" ? collectVueLocal(file) : collectJsLocal(file);
		}
	});

	// 处理中文，获取被删除的key值
	Object.keys(oldLocals).forEach((key) => {
		if (!locals.hasOwnProperty(key) && !isObject(oldLocals[key])) {
			const value = oldLocals[key];
			const md5 = Utils.Md5_16(value);
			if (md5 === key) {
				delKey[key] = value;
				// 删除已无用的中文
				delete oldLocals[key];
			}
		}
	});
	Object.assign(delKeyCache, delKey);
	console.log(chalk.blue(`\n 共 ${Object.keys(delKey).length}项需要清理！\n`));
	rootPath = path.join(process.cwd(), translateConfig.dir + translateConfig.file);
	let writeFileData = Object.assign({}, sourceLocal, {
		local: oldLocals,
		deleteLocal: delKeyCache
	});
	fs.writeFileSync(rootPath, `module.exports = ${JSON.stringify(writeFileData, null, "\t")};`, "utf8");
	spinner.succeed(chalk.green(" ✅ zh.js已清理完成"));

	// 同步导入所有国际化文件
	langList.forEach((lang) => {
		// 读取已经翻译的英文文件等
		if (fs.existsSync(translateConfig.dir + lang + ".js")) {
			processFile = path.join(process.cwd(), translateConfig.dir + lang + ".js");
			const langSourceData = require(processFile);
			if (langSourceData.hasOwnProperty("local")) {
				let localData = langSourceData.local;
				Object.keys(delKey).forEach((key) => {
					if (localData.hasOwnProperty(key)) {
						delete localData[key];
					}
				});
				let writeFileData = Object.assign({}, langSourceData, {
					local: localData
				})
				fs.writeFileSync(processFile, `module.exports = ${JSON.stringify(writeFileData, null, "\t")};`, "utf8");
				spinner.succeed(chalk.green(` ✅ ${lang}.js已清理完成`));
			} else {
				console.log(chalk.blue(`\n 共 ${lang}.js文件不存在local数据,跳过！\n`));
			}

		}
	});
};
