const chalk = require("chalk");
const path = require("path");
const fs = require("fs");
const Utils = require("./utils");
const FileProcess = require("./fileProcess");
let [oldFileLocal, i18nPath, i18nFile] = [];
let newLocals = {}
const isObject = (obj) => Object.prototype.toString.call(obj) === '[object Object]'

const diffObject = (oldObj, newObj) => {
	const result = {}
	let newKeys = Object.keys(newObj);;
	for (const key of newKeys) {

		const newVal = newObj[key];
		// 旧值存在这个 key 判断两个值是否相同
		if (oldObj.hasOwnProperty(key)) {
			const oldVal = oldObj[key];
			const areObjects = isObject(oldVal) && isObject(newVal);
			if (areObjects) {
				// 都是对象继续比较
				const tempResult = diffObject(oldVal, newVal)
				Object.assign(result, tempResult)
			} else if (oldVal != newVal) {
				// 新值和旧值不同
				result[key] = newVal
			}
		} else {
			result[key] = newVal
		}
	}

	return result;
}

/**
 * @description: 将国际化信息放到国际化资源文件中
 * @author: - wsz -
 * @param {*}
 */
const writeLocalToFile = () => {
	const i18nFilePath = path.join(i18nPath, i18nFile);
	const oldLocals = oldFileLocal.local
	// 新添加的国际化数据
	const newCollectLocals = diffObject(oldLocals, newLocals)
	// 有新增加的语言key
	if (Object.keys(newCollectLocals).length > 0) {
		const fileResult = Object.assign({}, oldFileLocal, {
			["new_" + new Date().getTime()]: newCollectLocals,
		});
		console.log(chalk.green("\n ⚠️ 发现 " + chalk.blue(Object.keys(newCollectLocals).length) + " 项新添加语言,请注意处理!\n"));
		// 写入到文件中，默认将新生成的内容，作为原有数据的一项
		fs.writeFileSync(i18nFilePath, `module.exports = ${JSON.stringify(fileResult, null, "\t")};`, "utf8");
	} else {
		console.log(chalk.yellow("\n ⚠️ 未发现新增的国际化数据\n"));
	}
};
/**
 * @description: 获取vue 文件语言key
 * @author: - wsz -
 * @param {*} file
 */
const collectVueLocal = (file) => {
	const processFile = path.relative(process.cwd(), file);
	const content = fs.readFileSync(file, "utf8");
	// // 获取模板部分
	const [, templateContent = ""] = content.match(/<template[^>]*>((.|\n)*)<\/template>/im) || [];
	FileProcess.generateTemplate(newLocals, templateContent);
	// 获取script部分
	const [, scriptContent = ""] = content.match(/<script[^>]*>((.|\n)*)<\/script>/im) || [];
	FileProcess.generateScript(newLocals, scriptContent);
	console.log(chalk.green('🌲 vue 收集完成 =>'), chalk.underline.yellow(processFile));
};

const collectJsLocal = (file) => {
	const processFile = path.relative(process.cwd(), file);
	const jsContent = fs.readFileSync(file, "utf8");
	FileProcess.generateScript(newLocals, jsContent);
	console.log(chalk.cyan('🪵 js 收集完成 =>'), chalk.underline.blue(processFile));
};

module.exports = function (src, { filepath, filename }) {
	const rootPath = path.join(process.cwd(), src);
	i18nFile = `${filename}.js`;
	i18nPath = path.join(process.cwd(), filepath);
	const i18nFilePath = path.join(i18nPath, i18nFile);
	if (fs.existsSync(i18nFilePath)) {
		oldFileLocal = require(i18nFilePath);
		// 判断一下是否符合规定的格式
		if (!oldFileLocal.hasOwnProperty('local')) {
			console.log(chalk.red(" ❌ 数据格式不正,请确认导出的数据包含local字段"));
			return
		}
		Utils.getAllFiles(rootPath).forEach((file) => {
			if (file.indexOf(i18nPath) < 0) {
				path.extname(file) === ".vue" ? collectVueLocal(file) : collectJsLocal(file);
			}
		});
		writeLocalToFile();
		console.log(chalk.green(" 💯 中文资源文件文件都已收集完毕，生成位置=>"), chalk.blue(path.join(i18nPath, i18nFile)));
	} else {
		console.log(" ❌ 获取文件目录失败，请检查是否输入正确的文件路径！")
	}

};
