const fs = require("fs");
const path = require("path");
const crypto = require("crypto");

/**
 * 创建深层目录
 * @param dirpath
 * @param mode
 */
exports.mkdirs = function (dirpath, mode) {
	mode = mode || "777";
	if (!fs.existsSync(dirpath)) {
		// 尝试创建父目录，然后再创建当前目录
		exports.mkdirs(path.dirname(dirpath), mode);
		fs.mkdirSync(dirpath, mode);
	}
};

/**
 * 删除深层目录
 * @param dirpath
 */
exports.rmdirs = function (dirpath) {
	try {
		fs.readdirSync(dirpath).forEach(function (filepath) {
			const state = fs.statSync(path.join(dirpath, filepath));
			if (state.isDirectory()) {
				exports.rmdirs(path.join(dirpath, filepath));
			} else {
				fs.unlinkSync(path.join(dirpath, filepath));
			}
		});
		fs.rmdirSync(dirpath);
	} catch (e) {
		console.log(e);
	}
};

/**
 * 获取MD5值
 * @param str
 * @return {string}
 * @constructor
 */
exports.Md5 = function (str) {
	return crypto.createHash("md5").update(str).digest("hex");
};
/**
 * 获取有道MD5值
 * @param str
 * @return {string}
 * @constructor
 */
exports.ydMd5 = function (str) {
	return crypto.SHA256(str).toString(CryptoJS.enc.Hex)
};

// 删除左右两端的空格
exports.trim = function (str) {
	return str.replace(/(^\s*)|(\s*$)/g, "");
};

/**
 * 获取16位的MD5值
 * @param str
 * @return {string}
 * @constructor
 */
exports.Md5_16 = function (str) {
	return exports.Md5(str).slice(8, 24);
};

/**
 * 获取指定目录下js和vue文件
 * @param dir
 * @return {[]}
 */
const getAllFiles = (exports.getAllFiles = (dir) => {
	const results = [];
	fs.readdirSync(dir).forEach((item) => {
		item = path.join(dir, item);
		if (fs.lstatSync(item).isDirectory()) {
			results.push(...getAllFiles(item));
		} else {
			if ([".vue", ".js", ".jsx", ".ts", ".tsx"].includes(path.extname(item).toLowerCase())) {
				results.push(item);
			}
		}
	});
	return results;
});

// 简单的语法校验
const grammar = (zh) => {
	if (zh.length < 8) {
		if (
			zh.indexOf("请") === 0 ||
			zh.indexOf("不能") > -1 ||
			zh.indexOf("必须为") > -1 ||
			zh.indexOf("确定") === 0 ||
			zh.indexOf("以内") > -1
		) {
			return false;
		}
		return true;
	}
	return false;
};
// 英语内介词的转化 和 html标签的转化
const preposition = (str) => {
	let val = str;
	val = val.replace(/ Of /g, " of ");
	val = val.replace(/ To /g, " to ");
	val = val.replace(/ And /g, " and ");
	val = val.replace(/ For /g, " for ");
	val = val.replace(/ Or /g, " or ");
	// html标签的转化
	val = val.replace(/< span /g, "<span ");
	val = val.replace(/< \/ span >/g, "</span>");
	val = val.replace(/< \/ span/g, "</span");
	val = val.replace(/span >/g, "span>");
	val = val.replace(/< p/g, "<p");
	val = val.replace(/< \/ p >/g, "</p>");
	val = val.replace(/< \/ p/g, "</p");
	val = val.replace(/p >/g, "p>");
	val = val.replace(/< i /g, "<i ");
	val = val.replace(/< \/ i >/g, "</i>");
	val = val.replace(/< \/ i/g, "</i");
	val = val.replace(/i >/g, "i>");
	val = val.replace(/< div /g, "<div ");
	val = val.replace(/< \/ div >/g, "</div>");
	val = val.replace(/< \/ div/g, "</div");
	val = val.replace(/div >/g, "div>");
	return val;
};
// 英文大小写转化匹配规则
exports.replaceStr = function (zh, str) {
	let value = str;
	// 全部转化
	if (grammar(zh)) {
		const arr = str.split(" ");
		for (let i = 0; i < arr.length; i++) {
			// 将首字母大写 拼接后面没有首字母的部分
			arr[i] = arr[i].charAt(0).toUpperCase() + arr[i].slice(1);
		}
		value = arr.join(" ");
	} else {
		// 单独首字母的转化
		value = str.substring(0, 1).toUpperCase() + str.substring(1);
	}
	return preposition(value);
};

// 获取语言数据
exports.getLocalData = (key, localData) => {
	let result = {};
	const keyPath = key.split(".")
	for (const p of keyPath) {
		result = Object.assign({}, localData[p])
	}
	return result
}